/* muzikQ.c 
  
 *  muzikQ - Curses-based audio player
 *
 *  Copyright (C) 2008 Tim A. Brown
 *
 *  This file is part of muzikQ.

    muzikQ is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    muzikQ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with muzikQ.  If not, see <http://www.gnu.org/licenses/>.

 */

#include "muzikQ.h"
#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#include <curses.h>
#include <sys/resource.h>
#include <sys/time.h>
#include <sys/types.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

Mix_Music *musicfile;			//Pointer to our music, in memory
int audio_rate = 44100;			//Frequency of audio playback
Uint16 audio_format = AUDIO_S16SYS; 	//Format of the audio we're playing
int audio_channels = 2;			//2 channels = stereo
int audio_buffers = 4096;		//Size of the audio buffers in memory
int musicPlaying = 0;  //is music playing?
int ky = 0, kx = 0, fe = 0;
time_t start, now, stop, diff; //various timing variables
struct tm *tm, *dm;
double pausediff = 0;
int t_diff = 0, d_diff = 0, restart = 0, seek_diff = 0, use_cdrom = 0;

void musicFinished();  //finished hook

int main (int argc, char *argv[])
{
  char buf[32];
  int i = 0, a = 0, y = 0;
  int ret = 0, s = 0;
  int songtmp = 0;
  
  //init SDL_mixer
  if ((SDL_Init (SDL_INIT_AUDIO) != 0) || !SDL_AudioDriverName (buf, 1))
    {
      fprintf (stderr, "Warning: Couldn't init SDL audio: %s\n",
	       SDL_GetError ());
      exit (1);
    }
  
    //Initialize SDL_mixer with our chosen audio settings
 if (Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers) != 0) 
	{
		printf("Unable to initialize audio: %s\n", Mix_GetError());
		exit (1);
	}
  
  // We don't want stderr stuff in out nice curses output
  stderrfp = freopen ("/dev/null", "w", stderr);
  defaultvol = 80;
  xterm_title = 0;
  delay_between_songs = 0;
  random_method = 2;
  parse_config ();
  //printf("\n%i\n", use_cdrom);
  if (use_cdrom)
    {
      if (SDL_InitSubSystem(SDL_INIT_CDROM) != 0)
        {
          printf("Unable to initialize CDROM: %s\n", SDL_GetError());
          exit(1);
        }
      //Open CD-ROM drive
      cdrom = SDL_CDOpen(0);
      if (cdrom == NULL) 
        {
            fprintf(stderr, "Couldn't open default CD-ROM drive: %s\n",
                        SDL_GetError());
            exit(2);
        }
      printf("Drives available: %d\n", Num_CDRoms());
    }
  
  s = decode_switches (argc, argv);
  
  term = getenv ("TERM");
  srand ((unsigned int) time (NULL));
  signal (SIGINT, end_program);
  signal (SIGWINCH, repaint);  //added for xterm functionality.
  
  i = 0;
  printf ("Loading ...\n");
  for (a = s; a < argc; a++)	/* for each playlist/mp3 provided */
    {
       load_playlist (argv[a]);
       get_playlist_times(a);
    } 
  getting_times = 1;
  
  if (use_cdrom)
    {
      songtmp = num_songs;
      num_songs = CD_Add_Playlist(songtmp);
    }
  
  init_screen ();
  getmaxyx(stdscr,ky,kx);
  getcwd (startdir, MAXPATHLEN); 
  //added above for being sure we are in the directory we started in
  
  if (num_songs == 0) show_filebrowser(); //added from submitted diff file.

  for (a = 0; a < num_songs; a++)
    play[a].mfile[kx - 15] = '\0';
  
  if (num_songs < 2)
    {
      random_play = 0;
      ret = y = 0;
    }
  else if (random_play)
    {
      generate_random_playlist ();
      ret = y = -1;
    }
  while (y < num_songs)
    {
      if (ret == -1)
	{
	  if (random_play)
	    {
	      y = rand () % num_randoms;
	      if (selected == current_song)
		{
		  selected = current_song = y = random_playlist[y];
		  pagetop = current_song - (ky - 9) / 2;
		}
	      else
		{
		  current_song = y = random_playlist[y];
                  //follow current song during random
                  selected = current_song;
		}
	    }
	  else
	    {
	      if (current_song == num_songs - 1)
		{
		  if (repeat_play)
		    y = 0;
		  else
		    end_program ();
		}
	      else
		{
		  y++;
		}
	      if (selected == current_song)
		selected = current_song = y;
	      else
		current_song = y;
	    }
	}
      else
	y = current_song = selected;
        ret = play_music (play[y]);
    }

  end_program ();
  return (0);
}

int decode_switches (int argc, char **argv)
{
  int c;
  while ((c = getopt_long (argc, argv,
			   "hvl::r::p:t::d:m:", long_options,
			   (int *) 0)) != EOF)
    {
      switch (c)
	{
	case 'v':
	  printf ("muzikQ %s\n", VERSION);
	  exit (0);

        case 'h':
	  usage (0);

	case 'l':
	  if (optarg == 0)
	    repeat_play = 1;
	  else
	    repeat_play = atoi (optarg);
	  break;

	case 'r':
	  if (optarg == 0)
	    random_play = 1;
	  else
	    random_play = atoi (optarg);
	  break;

	case 't':
	  if (optarg == 0)
	    xterm_title = 1;
	  else
	    xterm_title = atoi (optarg);
	  break;

	case 'd':
	  delay_between_songs = atoi (optarg) * 1000;
	  break;

	case 'm':
	  random_method = atoi (optarg);
	  break;

	case 'p':
	  strcpy ((char *) mkQplaylist, optarg);
	  break;
            
	default:
	  usage (0);
	}
    }
  return optind;
}

int play_music (struct PlayList mp3) {
    
  int ch = 0, fd = 0, http = 0, last_song = 0;
  int mmin = 0, msec = 0;
  struct PlayList tmpplay;
  
  getmaxyx(stdscr,ky,kx);
  
  if (strncmp (mp3.file, "http://", strlen ("http://")) == 0)
    {
      fd = http_open ((char *) mp3.file);
      
      http = 1;
    }

  if (play[current_song].format != 3)
      {
        musicfile = Mix_LoadMUS(play[current_song].file);
	if(musicfile == NULL) 
	{
		printf("Unable to load music file: %s\n", Mix_GetError());
		return 1;
	}
      }
 
  if (play[current_song].format == 3)
    {
        Play_Track(play[current_song].track);
    }
  
  if (play[current_song].format != 3)
    get_playlist_times (current_song);
  if (play[current_song].vol)
    vol = play[current_song].vol;
   else
    vol = defaultvol;
  //set volume
 if (play[current_song].format != 3)
    {
        Mix_VolumeMusic(vol);
        //play music
        if(Mix_PlayMusic(musicfile, 0) == -1) 
            {
		printf("Unable to play music file: %s\n", Mix_GetError());
		return 1;

            }
    }
  //start timer
  start = time(NULL);

  //music is now playing
  musicPlaying = 1;
	  
//Make sure that the musicFinished() function is called when the music stops playing
 if (play[current_song].format != 3)
    Mix_HookMusicFinished(musicFinished);
  //not paused =)
  is_paused = 0;
  if (!strcmp ((char *) term, (char *) "xterm") && xterm_title)
    {
      if (strlen (play[current_song].artist) == 0
	  && strlen (play[current_song].title) == 0)
	printf ("\033]2;muzikQ %s : %s\007", VERSION,
		play[current_song].mfile);
      else
	printf ("\033]2;muzikQ %s : %s - %s\007", VERSION,
		play[current_song].artist, play[current_song].title);
    }
 //Loop while music is playing
 //while (Mix_PlayingMusic() == 1)
  while (musicPlaying)
    {
     
     if (restart)
        {
            start = time(NULL);
            d_diff = 0;
            t_diff = 0;
            seek_diff = 0;
            restart = 0;
            return(-1);
        }
     if (!is_paused) 
        {
            now = time(NULL);
            diff = now - start;
            tm = localtime(&diff);
            //mhour = (tm->tm_hour * 60) * 60;
            mmin = tm->tm_min * 60;
            msec = tm->tm_sec;
            t_diff = (mmin + msec + d_diff); //total seconds;
        }
      while (play[q].seconds && q < num_songs && play[q].format != 3)
      if (q < num_songs)
	{
	  getting_times = 1;
	  get_playlist_times (q);
	  q++;
	}
      else if (random && random_method == 3 && getting_times)
        {
	  generate_random_playlist ();
	  getting_times = 0;
	} 
      show_main_win (t_diff);
      show_right_win ();
      ch = toupper(getch());
      if (ch == 0) break;
      switch(ch) {
          case KEY_F(4):
              if (playlist_sort == 1)
	        sort_playlist (5);
	      else
	        sort_playlist (1);
	      if (q != num_songs)
	        q = 0;
              break;
              
          case KEY_F(3):
              if (playlist_sort == 2)
                sort_playlist (6);
              else
                sort_playlist (2);
              if (q != num_songs)
                q = 0;
              break;
              
          case KEY_F(2):
              if (playlist_sort == 3)
                sort_playlist (7);
              else
                sort_playlist (3);
              if (q != num_songs)
                q = 0;
              break;
              
          case KEY_F(1):
              if (playlist_sort == 4)
                sort_playlist (8);
              else
                sort_playlist (4);
              if (q != num_songs)
                q = 0;
              break;
          case 32 :
              if (!is_paused)
                {
                    //pause playback
                    if (play[current_song].format != 3)
                        Mix_PauseMusic();
                    if (play[current_song].format == 3)
                        Pause_CD();
                    is_paused = 1;
                    musicPlaying = 1;
                    //record stop time
                    stop = time(NULL);
                    SDL_Delay (200);
                    /* pause a while to make sure the info gets updated */
                }
              else
                {
                    //Resume playback
                    if (play[current_song].format != 3)
                        Mix_ResumeMusic();
                    if (play[current_song].format == 3)
                        Resume_CD();
                    is_paused = 0;
                    musicPlaying = 1;
                    now = time(NULL);
                    pausediff = difftime(stop,start);
                    start = now - pausediff;
                    SDL_Delay (200);	
                    /* pause a while to make sure the info gets updated */
                }
              break;
              
          case '+':
              if (vol != 100)
                {
                    playlist_changes = 1;
                    play[current_song].vol = ++vol;
                    if (vol == defaultvol)
                        play[current_song].vol = 0;
                    //increase volume
                    Mix_VolumeMusic(vol);
                }
              break;
              
          case '-':
              if (vol)
                {
                    playlist_changes = 1;
                    play[current_song].vol = --vol;
                    if (vol == defaultvol)
                      play[current_song].vol = 0;
                    //decrease volume
                    Mix_VolumeMusic(vol);
                }
              break;
              
          case KEY_RIGHT:
              if (play[current_song].format == 3)
                  break;
              music_seek (KEY_RIGHT, diff);
              break;
              
          case KEY_LEFT:
              if (play[current_song].format == 3)
                  break;
              music_seek (KEY_LEFT, diff);
              break;
              
          case KEY_DOWN:
              if (selected < num_songs - 1) {
	        selected++;
                update_scrollbar();
              }
              break;
              
          case KEY_UP:
              if (selected) {
	        selected--;
                update_scrollbar();
              }
              break;
          
          case KEY_NPAGE:
              if (selected + 10 < num_songs - 1)
                {
                    selected += ky - 9;
                    pagetop += ky - 9;
                    if (selected > num_songs -1)
                        selected == num_songs -1;
                }
              else {
                     selected = num_songs - 1;
                }
              update_scrollbar();
              break;
              
          case KEY_PPAGE:
              if (selected - 10 >= 0)
                {
                    selected -= ky - 9;
                    pagetop -= ky - 9;
                }
              else
                {
                    selected = 0;
                }
              update_scrollbar();
              break;
              
          case KEY_HOME:
              selected = current_song;
              break;
          
          case KEY_END:
              selected = num_songs - 1;
              break;
              
          case ']':
              if (selected != 0) {
                  memcpy (&tmpplay, &play[selected], sizeof (struct PlayList));
                  memcpy (&play[selected], &play[selected - 1],
		  sizeof (struct PlayList));
                  memcpy (&play[selected - 1], &tmpplay, sizeof (struct PlayList));
                  if (current_song == selected)
                    current_song--;
                  else if (current_song == selected - 1)
                    current_song++;
                  selected--;
                  if (random_play)
                    generate_random_playlist ();
                  playlist_changes = 1;
              }
              break;
              
          case '[':
              if (num_songs > 1 && selected != num_songs - 1) {
                   memcpy (&tmpplay, &play[selected], sizeof (struct PlayList));
                   memcpy (&play[selected], &play[selected + 1],
		   sizeof (struct PlayList));
                   memcpy (&play[selected + 1], &tmpplay, sizeof (struct PlayList));
                   if (current_song == selected)
                        current_song++;
                   else if (current_song == selected + 1)
                        current_song--;
                   selected++;
                   if (random_play)
                        generate_random_playlist ();
                   playlist_changes = 1;
              }
              break;
              
          case 'R':
              if (!random_play)
                {
                    generate_random_playlist ();
                    random_play = 1;
                }
              else
                {
                    random_play = 0;
                }
              break;
              
          case 'A':
              show_filebrowser ();
              getting_times = 1;
              init_screen ();
              break;
              
          case 'L':
              if (!repeat_play)
                repeat_play = 1;
              else
                repeat_play = 0;
              break;
          
          case 'N':
              if (play[current_song].format == 3)
                {
                  Stop_CD();
                  SDL_Delay(10);
                }
              if (play[current_song].format != 3)
                {
                    Mix_HaltMusic();  //stop playback
                    Mix_FreeMusic(musicfile); //free mem
                }
              if (current_song = num_songs) 
                {
                  current_song = 0;
                  selected++;
              }  
              d_diff = 0;
              t_diff = 0;
              //musicPlaying = 0;
              return (-1);
              break;
              
          case 'H':
              show_help ();
              break;
              
          case 'Q':
              if (playlist_changes)
                if (confirm ("You have not saved your playlist", "Save it now?"))
                    fe = write_playlist ();
              if (play[current_song].format == 3)
                  Stop_CD();
              end_program ();
              break;
          
          case 10:
              if (play[current_song].format != 3)
                {
                    Mix_HaltMusic(); //stop playback
                    Mix_FreeMusic(musicfile); //release mem
                }
              if (play[current_song].format == 3)
                {
                  Stop_CD();
                }
              d_diff = 0;
              t_diff = 0;
              return (selected);
              break;
              
          case 'D':
              if (confirm ("", "Delete from playlist?"))
                {
                   if (selected == current_song) //check if playing
                     {
                       if (play[current_song].format != 3)
                        {
                            Mix_HaltMusic(); //halt playback
                            Mix_FreeMusic(musicfile); //release memory
                            
                        }
                       else Stop_CD();
                     }
                   musicPlaying = 0;
                   delete_song(selected);
                   num_songs--;
                   SDL_Delay(200);
                }
                if (random_play)
                  generate_random_playlist ();
              break;
              
          case '1':
              playlist_changes = 1;
              play[selected].rate = 1;
	      if (random_play)
                generate_random_playlist ();
              break;
          
          case '2':
              playlist_changes = 1;
              play[selected].rate = 2;
	      if (random_play)
                generate_random_playlist ();
              break;
          
          case '3':
              playlist_changes = 1;
              play[selected].rate = 3;
	      if (random_play)
                generate_random_playlist ();
              break;
              
          case '4':
              playlist_changes = 1;
              play[selected].rate = 4;
	      if (random_play)
                generate_random_playlist ();
              break;
              
          case '5':
              playlist_changes = 1;
              play[selected].rate = 5;
	      if (random_play)
                generate_random_playlist ();
              break;
              
          case '6':
              playlist_changes = 1;
              play[selected].rate = 6;
	      if (random_play)
                generate_random_playlist ();
              break;
              
          case '7':
              playlist_changes = 1;
              play[selected].rate = 7;
	      if (random_play)
                generate_random_playlist ();
              break;
              
          case '8':
              playlist_changes = 1;
              play[selected].rate = 8;
	      if (random_play)
                generate_random_playlist ();
              break;
              
          case '9':
              playlist_changes = 1;
              play[selected].rate = 9;
	      if (random_play)
                generate_random_playlist ();
              break;
          case 'E':
              show_tag_win ();
              break;
              
          case 'S':
              fe = write_playlist ();
              break;
              
          case '/':
              show_search_win ();
              break;
              
          case 'C':
              selected = current_song;
              pagetop = current_song - (ky - 9) / 2;
              break;
          default : break;
    }
 }   
  last_song = current_song;
  
  if (musicPlaying == 0) 
    {                                //if music not playing
        if (play[current_song].format != 3)
            {
                Mix_HaltMusic();            //halt playback
                Mix_FreeMusic(musicfile);  //free mem
            }
        if (play[current_song].format != 3)
            Stop_CD();
        pausediff = 0;            //reset variables for timing
        diff = 0;
        d_diff = 0;
        t_diff = 0;
        restart = 1;
        SDL_Delay(1);
        return(-1);
  }
  if (delay_between_songs)
    SDL_Delay (delay_between_songs);  //delay between playback
  return (-1);
}

void musicFinished()
{
	//Music is done!
	musicPlaying = 0;
}

void music_seek (int key, time_t current_time)
{   
    int thour = 0, tmin = 0, tsec = 0;
    int dhour = 0, dmin = 0, dsec, t_total, d_total;
      
    if (play[current_song].format == 3)
        return;
    seek_diff =0;
    now = time(NULL);
    tm = localtime(&now);
    dm = localtime(&diff);
    thour = tm->tm_hour;
    tmin = tm->tm_min;
    tsec = tm->tm_sec;
    dhour = dm->tm_hour;
    dmin = dm->tm_min;
    dsec = dm->tm_sec;
    t_total = ((thour / 60) / 60) + (tmin / 60) + tsec; //total seconds;
    d_total = ((dhour / 60) / 60) + (dmin / 60) + dsec; //total seconds;
    
    switch(key) {
        case KEY_LEFT:
            Mix_RewindMusic();
               seek_diff = d_diff - 10;
            if (seek_diff < 0)
                {   
                    seek_diff = 0;
                    restart = 1;
                }
            break;
        case KEY_RIGHT:
            Mix_PauseMusic();
            if (seek_diff + d_diff > play[current_song].seconds)
                {
                    restart = 1;
                }
            if (!restart) 
                seek_diff = t_total - d_total + 10;
            break;
    }
    freopen ("/dev/null", "w", stdout); //redirect stdout so SetMusicPosition() 
                                        //doesn't make a mess =)
    if (Mix_SetMusicPosition(seek_diff) == -1)
        printf("Mix_SetMusicPosition: %s\n", Mix_GetError());
    fclose (stdout);                    //close redirect
    freopen ("/dev/tty", "w", stdout); //reopen proper stdout =)
    SDL_Delay(1);
    if (key == KEY_RIGHT)
        d_diff = d_diff + seek_diff;
    if (key == KEY_LEFT)
        d_diff = seek_diff -6;
    SDL_Delay(1);
    if (key == KEY_RIGHT) 
      Mix_ResumeMusic();
}

char *stristr (const char *String, const char *Pattern)
{
  char *pptr, *sptr, *start;
  unsigned int slen, plen;

  for (start = (char *) String,
       pptr = (char *) Pattern,
       slen = strlen (String), plen = strlen (Pattern);
       /* while string length not shorter than pattern length */
       slen >= plen; start++, slen--)
    {
      /* find start of pattern in string */
      while (toupper (*start) != toupper (*Pattern))
	{
	  start++;
	  slen--;

	  /* if pattern longer than string */

	  if (slen < plen)
	    return (NULL);
	}

      sptr = start;
      pptr = (char *) Pattern;

      while (toupper (*sptr) == toupper (*pptr))
	{
	  sptr++;
	  pptr++;

	  /* if end of pattern then pattern was found */

	  if ('\0' == *pptr)
	    return (start);
	}
    }
  return (NULL);
}

void usage (int status)
{
  printf ("muzikQ - curses based audio player\n");
  printf ("Usage: muzikQ [OPTION ...] FILE1 [FILE2 ...]\n");
  printf ("\
  Options:\n\
  -h, --help                 display this help and exit\n\
  -v, --version              output version information and exit\n\
  -r, --random               start in random play mode (0/1, default=1)\n\
  -l, --loop                 start in loop play mode (0/1, default=1)\n\
  -t, --title                set xterm title (0/1, default=1)\n\
  -d, --delay                delay between songs (in seconds)\n\
  -m, --rmethod              method used for random play (1/2/3)\n\
  -p, --playlist             default playlist to save to\n\
");
  exit (status);
}

void end_program ()
{
  endwin ();
  fclose (stderrfp);
  if (play[current_song].format != 3)
    {
        //Stop music
        Mix_HaltMusic();
        //Release the memory allocated to our music
        Mix_FreeMusic(musicfile);
    }
  //Need to make sure that SDL_mixer and SDL have a chance to clean up
  Mix_CloseAudio();
  SDL_CDClose(cdrom); //close cdrom
  SDL_Quit ();
  exit (0);
}

char *trim(char *str)
{
    char ptr[strlen(str)+1];
    int z, j = 0;
    for(z=0; (z < strlen(str)) && (str[z] != '\0'); z++)
    {
        if (str[z] != ' ' && str[z] != '\t')
        {
            ptr[j++]=str[z];
        }
    }
    //ptr[strlen(ptr)+1]='\0';
    //str = ptr;
    return(str);
} 

void parse_config ()
{
  char *home, conffile[256], line[256], *var, *val;
  FILE *infile;

  home = getenv ("HOME");
  sprintf ((char *) &conffile, "%s/.muzikQrc", home);
  infile = fopen (conffile, "r+");
  if (infile)
    {
      while (fgets (line, 255, infile))
	if (line[0] != '#' && line[0] != ' ' && line[0] != '\n')
	  {
	    var = strtok ((char *) &line, " = ");
	    val = strtok (NULL, " = ");
	    val[strlen (val) - 1] = '\0';

	    if (strcasecmp (var, "random") == 0)
	      {
		if (atoi (val) == 0 || atoi (val) == 1)
		  {
		    random_play = atoi (val);
		  }
		else
		  {
		    printf
		      ("config: random can only take a value of 0 or 1.\n");
		    exit (1);
		  }
	      }
	    else if (strcasecmp (var, "loop") == 0)
	      {
		if (atoi (val) == 0 || atoi (val) == 1)
		  {
		    repeat_play = atoi (val);
		  }
		else
		  {
		    printf
		      ("config: loop can only take a value of 0 or 1.\n");
		    exit (1);
		  }
	      }
	    else if (strcasecmp (var, "set_xterm_title") == 0)
	      {
		if (atoi (val) == 0 || atoi (val) == 1)
		  {
		    xterm_title = atoi (val);
		  }
		else
		  {
		    printf
		      ("config: set_xterm_title can only take a value of 0 or 1.\n");
		    exit (1);
		  }
	      }
	    else if (strcasecmp (var, "random_method") == 0)
	      {
		if (0 < atoi (val) < 4)
		  {
		    random_method = atoi (val);
		  }
		else
		  {
		    printf
		      ("config: random_method can only take a value between 1 and 3.\n");
		    exit (1);
		  }
	      }
	    else if (strcasecmp (var, "delay_between_songs") == 0)
	      {
		if (0 < atoi (val) < 10)
		  {
		    delay_between_songs = atoi (val) * 1000;
		  }
		else
		  {
		    printf
		      ("config: delay_between_songs can only take a value between 0 or 10.\n");
		    exit (1);
		  }
	      }
	    else if (strcasecmp (var, "volume") == 0)
	      {
		if (atoi (val) >= 0 && atoi (val) <= 100)
		  {
		    defaultvol = atoi (val);
		  }
		else
		  {
		    printf
		      ("config: volume can only take a value between 0 and 100.\n");
		    exit (1);
		  }
	      }
            else if (strcasecmp (var, "cdrom") == 0)
	      {
		if (atoi (val) == 0 || atoi (val) == 1)
		  {
		    use_cdrom = atoi (val);
		  }
              }
            else if (strcasecmp (var, "buffers") == 0)
	      {
		if (atoi (val) >= 0 && atoi (val) <= 4096)
		  {
		    audio_buffers = atoi (val);
		  }
              }
            else if (strcasecmp (var, "channels") == 0)
	      {
		if (atoi (val) >= 0 && atoi (val) <= 8)
		  {
		    audio_channels = atoi (val);
		  }
              }
	    else
	      {
		printf ("config: Unkown variable (%s)\n", var);
		exit (1);
	      }
	  }
      fclose (infile);
    }
}