/*
    muzikQ - curses based audio player
 *
    Copyright (C) 2008 Tim A. Brown 
 * 
 *  This file is part of muzikQ.

    muzikQ is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    muzikQ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with muzikQ.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef _MUZIKQ_HEADER__
#define _MUZIKQ_HEADER__
#include <SDL/SDL_mixer.h>
#include <SDL/SDL.h>
#include <stdio.h>
#include <sys/dir.h>
#include <curses.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#ifdef __linux__
#include <getopt.h>
#else
struct option
{
  char *name;
  int has_arg;
  int *flag;
  int val;
};
# define no_argument 0
# define required_argument 1
# define optional_argument 2
#endif
#define BUFLEN 200
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

struct PlayList
{
  int format;        // 1=mp3, 2=ogg, 3=cdrom  
  char file[256];    
  char mfile[256]; 
  int minutes;  
  int seconds;
  int sample;
  int rate;
  int bitrate;
  int vol;
  char title[31];
  char artist[31]; 
  char type[31];
  char year[5];  //need to add to edit
  char genre[31]; // need to add to edit
  char album[31]; // need to add to edit
  char comment[256]; //need to add to edit
  int track;  //need to add to edit
  int channels; 
  int size;
  //Uint32 size;
}
play[32000];

static struct option const long_options[] = {
  {"help", no_argument, 0, 'h'},
  {"version", no_argument, 0, 'v'},
  {"loop", optional_argument, 0, 'l'},
  {"random", optional_argument, 0, 'r'},
  {"title", optional_argument, 0, 't'},
  {"delay", required_argument, 0, 'd'},
  {"rmethod", required_argument, 0, 'm'},
  {"playlist", 1, 0, 'p'},
  {NULL, 0, NULL, 0}
};

int num_songs, pagetop, vol, defaultvol, q, is_paused,
    playlist_changes, num_randoms, random_playlist[320000], selected, 
    random_play, repeat_play, current_song, playlist_sort, 
    xterm_title, delay_between_songs, random_method, getting_times;
unsigned long total_time, total_size;
char startdir[MAXPATHLEN], mkQplaylist[100], *term, searchfor[27];
FILE *stderrfp;
WINDOW *main_win, *left_win, *right_win;
SDL_CD *cdrom;
CDstatus status;

void end_program ();
void show_right_win();
void show_main_win(float current_time);
int show_playlist_win ();
void get_playlist_times (int q);
void show_help ();
void music_seek (int key, time_t current_time);
void init_screen();
void parse_config ();
void repaint ();
void load_playlist (char *argv);
int search_header (FILE * fp);
int delete_song (int song);
int play_music (struct PlayList mp3);
int decode_switches (int argc, char **argv);
int confirm (char *statement, char *question);
void usage (int status);
void show_search_win ();
int write_playlist ();
void generate_random_playlist ();
void show_tag_win ();
void show_filebrowser ();
void sort_playlist (int what);
char *stristr (const char *String, const char *Pattern);
int sort_playlist_rate (const struct PlayList *c1, const struct PlayList *c2);
int sort_playlist_time (const struct PlayList *c1, const struct PlayList *c2);
int sort_playlist_artist (const struct PlayList *c1,
			  const struct PlayList *c2);
int sort_playlist_title (const struct PlayList *c1,
			 const struct PlayList *c2);
int sort_playlist_rate_rev (const struct PlayList *c1,
			    const struct PlayList *c2);
int sort_playlist_time_rev (const struct PlayList *c1,
			    const struct PlayList *c2);
int sort_playlist_artist_rev (const struct PlayList *c1,
			      const struct PlayList *c2);
int sort_playlist_title_rev (const struct PlayList *c1,
			     const struct PlayList *c2);
int tcp_open (char *address, int port);
int http_open (char *arg);
char *itoa(int value);
int getTags(int x);
int saveTags (int x);
void Stop_CD();
void Pause_CD();
void Resume_CD();
void Play_Track(int track);
int Num_CDRoms();
CDstatus CD_Status();
int CD_Add_Playlist(int songs);
#endif
