/* Get tags utilizing taglib 
 *
    muzikQ - Curses-based audio player
 *
 *  Copyright (C) 2008 Tim A. Brown
 *
 *  This file is part of muzikQ.

    muzikQ is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    muzikQ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with muzikQ.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "muzikQ.h"
#include <stdio.h>
#include <string.h>
#include <taglib/tag_c.h>

/*typedef enum {
  TagLib_File_MPEG,
  TagLib_File_OggVorbis,
  TagLib_File_FLAC,
  TagLib_File_MPC,
  TagLib_File_OggFlac,
  TagLib_File_WavPack,
  TagLib_File_Speex,
  TagLib_File_TrueAudio
} TagLib_File_Type; */


 int i = 0;
 TagLib_File *file;
 TagLib_Tag *tag;
 const TagLib_AudioProperties *properties;

 //below ruthlessly taken from http://www.techinterviews.com/?p=318
  //slightly modified =)
  char *itoa(int value)
    {
        int count = 0, /* number of characters in string */
        i = 0, /* loop control variable */
        sign = 0; /* determine if the value is negative */
        char *ptr, /* temporary pointer, index into string */
        *string, /* return value */
        *temp; /* temporary string array */

        //count = 0;
        if ((sign = value) < 0) /* assign value to sign, if negative */
        { /* keep track and invert value */
            value = -value;
            count++; /* increment count */
        }

        /* allocate INTSIZE plus 2 bytes (sign and NULL) */
        temp = (char *) malloc(sizeof (int) + 2);
        if (temp == NULL)
           {
                return(NULL);
           }
        memset(temp,'\0', sizeof (int) + 2);
        string = (char *) malloc(sizeof (int) + 2);
        if (string == NULL)
            {
                return(NULL);
            }
        memset(string,'\0', sizeof (int) + 2);
        ptr = string; /* set temporary ptr to string */
        
        /*--------------------------------------------------------------------+
        | NOTE: This process reverses the order of an integer, ie: |
        | value = -1234 equates to: char [4321-] |
        | Reorder the values using for {} loop below |
        +--------------------------------------------------------------------*/
        
        do {
            *temp++ = value % 10 + '0'; /* obtain modulus and or with '0' */
            count++; /* increment count, track iterations*/
           } while (( value /= 10) >0);

        if (sign < 0) /* add '-' when sign is negative */
            *temp++ = '-';

        *temp-- = '\0'; /* ensure null terminated and point */
        /* to last char in array */

        /*--------------------------------------------------------------------+
        | reorder the resulting char *string: |
        | temp - points to the last char in the temporary array |
        | ptr - points to the first element in the string array |
        +--------------------------------------------------------------------*/
        
        for (i = 0; i < count; i++, temp--, ptr++)
            {
                memcpy(ptr,temp,sizeof(char));
            }
        ptr = NULL;
        temp = NULL;
        //string = NULL;
        i = 0;
        count = 0;
        sign = 0;
        return(string);
    }
 
  //get tags
  int getTags(int x)
  {
      taglib_set_strings_unicode(FALSE);
      file = taglib_file_new(play[x].file);
      if(file != NULL && taglib_file_is_valid(file))
      {
          tag = taglib_file_tag(file);
          properties = taglib_file_audioproperties(file);
          memset (play[x].title, 0, 31 * sizeof (char));
          strncpy(play[x].title, taglib_tag_title(tag), 30);
          memset (play[x].artist, 0, 31 * sizeof (char));
          strncpy(play[x].artist, taglib_tag_artist(tag), 30);
          strncpy(play[x].album, taglib_tag_album(tag), 30);
          memset (play[x].year, 0, 5 * sizeof (char));
          strncpy(play[x].year, itoa(taglib_tag_year(tag)), 4);
          memset (play[x].comment, 0, 256 * sizeof (char));
          strncpy(play[x].comment, taglib_tag_comment(tag), 30);
          play[x].track = taglib_tag_track(tag);
          strncpy(play[x].genre, taglib_tag_genre(tag), 30);
          play[x].seconds = taglib_audioproperties_length(properties);
          play[x].minutes = (taglib_audioproperties_length(properties) - play[x].seconds) / 60;
          play[x].sample = taglib_audioproperties_samplerate(properties);
          play[x].bitrate = taglib_audioproperties_bitrate(properties);
          play[x].channels = taglib_audioproperties_channels(properties);
          taglib_file_free(file);
          taglib_tag_free_strings();
      } else printf("\ngetTags() failed.\n");
    return 0;   
  }
  
  //save tags
  int saveTags (int x)
    {
      int year;
      taglib_set_strings_unicode(FALSE);
      switch (play[x].format) {
          case 1:
              file = taglib_file_new_type(play[x].file, TagLib_File_MPEG);
              break;
          case 2:
              file = taglib_file_new_type(play[x].file, TagLib_File_OggVorbis);
              break;
          case 3 : break; //cdrom
          default: break;
              
      }      
      tag = taglib_file_tag(file);
      if(file != NULL && taglib_file_is_valid(file))
        {
            taglib_tag_set_title(tag, play[x].title);
            taglib_tag_set_artist(tag, play[x].artist);
            taglib_tag_set_album(tag, play[x].album);
            taglib_tag_set_comment(tag, play[x].comment);
            taglib_tag_set_genre(tag, play[x].genre);
            taglib_tag_set_year(tag, atoi(play[x].year));
            taglib_tag_set_track(tag, play[x].track);
            taglib_file_save(file);
            taglib_file_free(file);
            taglib_tag_free_strings();
       } else printf("\nsaveTags() failed.\n");
      return 0;
    }

  